aboutsummaryrefslogtreecommitdiff
diff options
context:
space:
mode:
authorTom Lane <tgl@sss.pgh.pa.us>2023-04-15 12:01:39 -0400
committerTom Lane <tgl@sss.pgh.pa.us>2023-04-15 12:01:39 -0400
commit9b104a27c72754e9a6a8043ad7e710c8c386a8db (patch)
tree6ce329829923ea26af99bffeeffaa1cc4dc87598
parente9884e9921862d2470bbccf39e3ed140a325afd3 (diff)
downloadpostgresql-9b104a27c72754e9a6a8043ad7e710c8c386a8db.tar.gz
postgresql-9b104a27c72754e9a6a8043ad7e710c8c386a8db.zip
Fix assignment to array of domain over composite, redux.
Commit 3e310d837 taught isAssignmentIndirectionExpr() to look through CoerceToDomain nodes. That's not sufficient, because since commit 04fe805a1 it's been possible for the planner to simplify CoerceToDomain to RelabelType when the domain has no constraints to enforce. So we need to look through RelabelType too. Per bug #17897 from Alexander Lakhin. Although 3e310d837 was back-patched to v11, it seems sufficient to apply this change to v12 and later, since 04fe805a1 came in in v12. Dmitry Dolgov Discussion: https://postgr.es/m/17897-4216c546c3874044@postgresql.org
-rw-r--r--src/backend/executor/execExpr.c10
-rw-r--r--src/test/regress/expected/domain.out9
-rw-r--r--src/test/regress/sql/domain.sql4
3 files changed, 21 insertions, 2 deletions
diff --git a/src/test/regress/expected/domain.out b/src/test/regress/expected/domain.out
index 8af2b89680a..a9a21ff5b95 100644
--- a/src/test/regress/expected/domain.out
+++ b/src/test/regress/expected/domain.out
@@ -556,6 +556,15 @@ table dcomptable;
{"(1,5)"}
(1 row)
+-- if there's no constraints, a different code path is taken:
+alter domain dcomptype drop constraint dcomptype_check;
+update dcomptable set f1[1].cf1 = -1; -- now ok
+table dcomptable;
+ f1
+------------
+ {"(-1,5)"}
+(1 row)
+
drop table dcomptable;
drop type comptype cascade;
NOTICE: drop cascades to type dcomptype
diff --git a/src/test/regress/sql/domain.sql b/src/test/regress/sql/domain.sql
index f2ca1fb675a..b735d9bfe19 100644
--- a/src/test/regress/sql/domain.sql
+++ b/src/test/regress/sql/domain.sql
@@ -288,6 +288,10 @@ table dcomptable;
update dcomptable set f1[1].cf1 = -1; -- fail
update dcomptable set f1[1].cf1 = 1;
table dcomptable;
+-- if there's no constraints, a different code path is taken:
+alter domain dcomptype drop constraint dcomptype_check;
+update dcomptable set f1[1].cf1 = -1; -- now ok
+table dcomptable;
drop table dcomptable;
drop type comptype cascade;
diff --git a/src/backend/executor/execExpr.c b/src/backend/executor/execExpr.c
index e2916e35c81..e6053482834 100644
--- a/src/backend/executor/execExpr.c
+++ b/src/backend/executor/execExpr.c
@@ -3132,8 +3132,8 @@ ExecInitSubscriptingRef(ExprEvalStep *scratch, SubscriptingRef *sbsref,
* trees in which each level of assignment has its own CaseTestExpr, and the
* recursive structure appears within the newvals or refassgnexpr field.
* There is an exception, though: if the array is an array-of-domain, we will
* have a CoerceToDomain as the refassgnexpr, and we need to be able to look
* through that.
*/
static bool
isAssignmentIndirectionExpr(Expr *expr)
@@ -3160,6 +3160,12 @@ isAssignmentIndirectionExpr(Expr *expr)
return isAssignmentIndirectionExpr(cd->arg);
}
+ else if (IsA(expr, RelabelType))
+ {
+ RelabelType *r = (RelabelType *) expr;
+
+ return isAssignmentIndirectionExpr(r->arg);
+ }
return false;
}
@@ -3132,8 +3132,8 @@ ExecInitSubscriptingRef(ExprEvalStep *scratch, SubscriptingRef *sbsref,
* recursive structure appears within the newvals or refassgnexpr field.
* There is an exception, though: if the array is an array-of-domain, we will
- * have a CoerceToDomain as the refassgnexpr, and we need to be able to look
- * through that.
+ * have a CoerceToDomain or RelabelType as the refassgnexpr, and we need to
+ * be able to look through that.
*/
static bool